<?php

/**
 * The file that holds a CartModifications class.
 *
 * @package OrderProtectionWp\Cart
 */

declare(strict_types=1);

namespace OrderProtectionWp\Cart;

use OrderProtectionWp\Manifest\Manifest;
use OrderProtectionWp\OrderProtectionApi\FetchOrderProtectionRatesInterface;
use OrderProtectionWp\Settings\PluginSettings;
use OrderProtectionWpVendor\EightshiftLibs\Helpers\Components;
use OrderProtectionWpVendor\EightshiftLibs\Services\ServiceInterface;
use OrderProtectionWp\Helpers\Cart;
use WC_Cart;

/**
 * CartModifications class.
 *
 * @since 1.0.0
 */
class CartModifications implements ServiceInterface
{
	private Cart $cartHelper;

	public function __construct(Cart $cartHelper)
	{
		$this->cartHelper = $cartHelper;
	}

	/**
	 * Register all the hooks
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function register(): void
	{
		add_filter('orderprotection_fee_widget_price', [$this, 'getPriceForFeeWidget'], 10);
		add_filter('orderprotection_fee_applied', [$this, 'checkIfFeeIsAppliedOnCart'], 10);
		add_action('woocommerce_cart_totals_after_order_total', [$this, 'addOrderProtectionWidgetToCart'], 10);
		add_filter('woocommerce_form_field', [$this, 'removeOptionalLabelFromOrderProtectionField'], 10, 2);
		add_action('woocommerce_cart_calculate_fees', [$this, 'addOrderprotectionFee'], 80);
		add_action('wp_footer', [$this, 'removeOrderprotectionFee']);
	}

	/**
	 * Get the price for the fee widget.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function getPriceForFeeWidget(): string
	{
		$cart = WC()->cart;
		$settings = $this->cartHelper->getSettings();
		$currentAmount = $this->cartHelper->getFeeInCart();

		$opFee = $settings['fee'];

		// if OP is in the cart but it does not match what it should be we will set the fee here

		if ($currentAmount > 0) {
			if ($opFee !== $currentAmount) {
				$this->cartHelper->addProtection();
			}
		}

		if (!PluginSettings::isOrderprotectionActive()) {
			$opFee = '0.00';
		}

		return wp_strip_all_tags(wc_price($opFee));
	}

	/**
	 * Check if the fee is applied on the cart.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	public function checkIfFeeIsAppliedOnCart(): bool
	{
		return $this->cartHelper->protectionAdded();
	}

	/**
	 * View render for the cart totals form
	 *
	 * @return void
	 */
	public function addOrderProtectionWidgetToCart(): void
	{
		if (!PluginSettings::isOrderprotectionActive()) {
			return;
		}

		// Forcing a reload of the page this is hit or miss if it works when reload is not set to true
		$this->cartHelper->renderWidget(true);
	}

	/**
	 * Remove "(optional)" text from the field.
	 *
	 * @since 1.0.0
	 *
	 * @param string $field Field as a string.
	 * @param string $key The Key of the field.
	 *
	 * @return string $field Returns the field string, once we have removed "(optional)".
	 */
	public function removeOptionalLabelFromOrderProtectionField(string $field, string $key): string
	{
		if ($key === Cart::FEE_ID) {
			$optional = '&nbsp;<span class="optional">(' . esc_html__('optional', 'orderprotection') . ')</span>';
			$field = str_replace($optional, '', $field);
		}

		return $field;
	}

	/**
	 * Add the OrderProtection fee based on the subtotal.
	 *
	 * @since 1.0.0
	 *
	 * @param object $cart WooCommerce Cart.
	 */
	public function addOrderprotectionFee(object $cart): void
	{
		if (is_admin() && !defined('DOING_AJAX')) {
			return;
		}

		if (!PluginSettings::isOrderprotectionActive()) {
			return;
		}

		if (!($cart instanceof WC_Cart)) {
			return;
		}

		if (!(bool) $this->cartHelper->protectionAdded()) {
			return;
		}

		$this->cartHelper->addProtection();
	}


	public function getCurrentOrderProtectionFeeAmount(): float
	{
		return $this->cartHelper->getFeeInCart();
	}

	public function removeOrderprotectionFee(): void
	{
		if (!is_wc_endpoint_url('order-received') || !$this->cartHelper->protectionAdded()) {
			return;
		}

		$this->cartHelper->removeProtection();
	}

	public function getFee(): array
	{
		$response = $this->cartHelper->getSettings();

		return $response['fee'];
	}

	public function isHidden(): bool
	{
		$response = $this->cartHelper->getSettings();

		return $response['hidden'];
	}

	public function defaultOn(): bool
	{
		$response = $this->cartHelper->getSettings();

		return $response['default_on'];
	}
}
