<?php

/**
 * File that holds encryption class
 *
 * @package OrderProtectionWp\Encryption
 */

declare(strict_types=1);

namespace OrderProtectionWp\Encryption;

use Exception;

/**
 * Encryption class
 *
 * Used to encrypt and decrypt sensitive data.
 *
 * @since 1.0.0
 */
class Encryptor implements Cryptor
{
	/**
	 * Encryption method used for encryption and decryption.
	 *
	 * @var string
	 */
	private string $encryptionMethod = 'AES-256-CBC';

	/**
	 * Key used for encryption and decryption.
	 *
	 * @var string
	 */
	private string $key;

	/**
	 * Initialization vector used for encryption and decryption.
	 *
	 * @var string
	 */
	private string $initVect;

	/**
	 * Class constructor
	 *
	 * Initialize vector used for encryption and decryption.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		/*
		 * Note about the secure passphrase.
		 * Users can define their own encryption key by defining OP_ENCRYPTION_KEY constant
		 * in wp-config.php file.
		 * If they don't set that, then the SECURE_AUTH_KEY salt will be used.
		 * This is less error-prone, because salts can be rotated.
		 * If the salt isn't set, a random string will be used, which is the least secure option.
		 */
		$securePassphrase = defined('OP_ENCRYPTION_KEY') ? OP_ENCRYPTION_KEY :
			(defined('SECURE_AUTH_KEY') ? SECURE_AUTH_KEY : 'EC<qy34?&5]!U{|xlago2rzgiS%[<^v8Zy2');

		$this->initVect = \substr(\hash('sha256', $securePassphrase), 0, 16);
	}

	/**
	 * Encrypt the given value
	 *
	 * @since 1.0.0
	 *
	 * @param string $value Value to encrypt.
	 *
	 * @return string Encrypted string.
	 * @throws Exception Throws exception in the case the encryption key is not set.
	 */
	public function encrypt(string $value): string
	{
		if (!isset($this->key)) {
			throw new Exception(esc_html__('Encryption key is not set', 'orderprotection-wp'));
		}

		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_encode
		return \base64_encode((string) \openssl_encrypt($value, $this->encryptionMethod, $this->key, 0, $this->initVect));
	}

	/**
	 * Decrypt the given value
	 *
	 * @since 1.0.0
	 *
	 * @param string $payload Value to decrypt.
	 *
	 * @throws Exception Throws exception in the case the encryption key is not set.
	 *
	 * @return mixed Decrypted value.
	 */
	public function decrypt(string $payload)
	{
		if (!isset($this->key)) {
			throw new Exception(esc_html__('Encryption key is not set', 'orderprotection-wp'));
		}

		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.obfuscation_base64_decode
		return \openssl_decrypt(\base64_decode($payload), $this->encryptionMethod, $this->key, 0, $this->initVect);
	}

	/**
	 * Get the encryption key that the encrypter is currently using
	 *
	 * @since 1.0.0
	 *
	 * @throws Exception Throws exception in the case the encryption key is not set.
	 *
	 * @return string Key used for encryption.
	 */
	public function getKey(): string
	{
		return $this->key;
	}

	/**
	 * Set the encryption key that the encrypter will use
	 *
	 * @since 1.0.0
	 *
	 * @param string|null $value Optional value to set the key with.
	 *
	 * @return void
	 */
	public function setKey(?string $value = null): void
	{
		if (!is_null($value)) {
			$this->key = $value;
			return;
		}

		$secretKey = \wp_salt();
		$this->key = \hash('sha256', $secretKey);
	}
}
