<?php

/**
 * The Frontend Enqueue specific functionality.
 *
 * @package OrderProtectionWp\Enqueue\Theme
 */

declare(strict_types=1);

namespace OrderProtectionWp\Enqueue\Front;

use OrderProtectionWp\Cart\CartFeeAddition;
use OrderProtectionWp\Cart\CartFeeInitialization;
use OrderProtectionWp\Config\Config;
use OrderProtectionWp\Helpers\Cart;
use OrderProtectionWpVendor\EightshiftLibs\Manifest\ManifestInterface;
use OrderProtectionWpVendor\EightshiftLibs\Enqueue\Theme\AbstractEnqueueTheme;

/**
 * Class EnqueueFront
 *
 * @since 1.0.0
 *
 * Class that holds all the front-end related enqueues.
 */
class EnqueueFront extends AbstractEnqueueTheme
{
    private Cart $cartHelper;
	/**
	 * Create a new class instance.
	 *
	 * @param ManifestInterface $manifest Inject manifest which holds data about assets from manifest.json.
	 *
	 * @since 1.0.0
	 */
	public function __construct(ManifestInterface $manifest, Cart $cart)
	{
        $this->manifest = $manifest;
        $this->cartHelper = $cart;
	}

	/**
	 * Register all the hooks.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function register(): void
	{
		\add_action('wp_enqueue_scripts', [$this, 'enqueueStyles'], 10);
		\add_action('wp_enqueue_scripts', [$this, 'enqueueScripts']);
		\add_action('woocommerce_review_order_before_payment', [$this, 'addWidgetToCheckout']);
	}

	/**
	 * Method that returns assets name used to prefix asset handlers.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function getAssetsPrefix(): string
	{
		return Config::getProjectName();
	}

	public function addWidgetToCheckout() {
        $this->cartHelper->renderWidget(true);
	}

	/**
	 * Method that returns assets version for versioning asset handlers.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function getAssetsVersion(): string
	{
		return Config::getProjectVersion();
	}

	/**
	 * Get script localizations
	 *
	 * * Example: $localization_array => [
	 *  'localizationHandler' => [
	 *      'someValue'    => \esc_html__( 'Hi there!', 'text-domain' ),
	 *      'anotherValue' => $variableValue,
	 *  ]
	 * ];
	 *
	 * @link https://developer.wordpress.org/reference/functions/wp_localize_script/
	 *
	 * @since 1.0.0
	 *
	 * @return array<string, mixed> Key value pair of different localizations.
	 */
	protected function getLocalizations(): array
	{
		return [
			'orderProtectionAjaxObject' => [
				'storeUrl' => site_url(),
				'ajaxUrl' => admin_url('admin-ajax.php'),
				'feeUpdateCallback' => CartFeeAddition::AJAX_CALLBACK_HANDLER,
				'feeInitCallback' => CartFeeInitialization::AJAX_CALLBACK_HANDLER,
				'nonce' => wp_create_nonce(CartFeeAddition::NONCE_ACTION),
				'nonceName' => CartFeeAddition::NONCE_NAME,
				'feeApplied' => CartFeeAddition::ORDER_PROTECTION_FEE_APPLIED_STRING,
				'errorMessage' => esc_html__(
					'An error occurred with OrderProtection, please try again.',
					'orderprotection-wp'
				),
			]
		];
	}

	/**
	 * Get frontend script dependencies
	 *
	 * @link https://developer.wordpress.org/reference/functions/wp_enqueue_script/#default-scripts-included-and-registered-by-wordpress
	 *
	 * @return array<int, string> List of all the script dependencies.
	 */
	protected function getFrontendScriptDependencies(): array
	{
		return ['wp-plugins', 'wp-data', 'wp-element'];
	}
}
