<?php

/**
 * Custom exception class for OrderProtection
 *
 * @package OrderProtectionWp\Exceptions
 */

declare(strict_types=1);

namespace OrderProtectionWp\Exceptions;

use Exception;
use OrderProtectionWp\Settings\PluginSettings;

/**
 * OrderProtectionException class.
 *
 * This class automatically sends error logs to the OrderProtection API when thrown.
 * It extends the base PHP Exception class and handles remote logging.
 */
class OrderProtectionException extends Exception
{
    private const LOG_ENDPOINT = 'https://api.production.orderprotection.com/v1/integrations/logs/create';

    /**
     * Constructor for OrderProtectionException
     *
     * @param string $message The exception message
     * @param int $code The exception code
     * @param Exception|null $previous The previous exception used for the exception chaining
     */
    public function __construct(string $message = "", int $code = 0, ?Exception $previous = null)
    {
        parent::__construct($message, $code, $previous);
        $this->sendLogToApi();
    }

    /**
     * Sends the error log to the OrderProtection API
     * This is a fire-and-forget request that doesn't wait for a response
     */
    private function sendLogToApi(): void
    {
        $storeUrl = get_option(PluginSettings::OPTION_STORE_URL, '');
		$pluginIsLive = get_option(PluginSettings::isOrderprotectionActive());

		if (!$pluginIsLive) {
			return;
		}

        // Get plugin data to access version
        $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/orderprotection/orderprotection.php');
        $plugin_version = $plugin_data['Version'] ?? 'unknown';

        $logData = [
            'service' => 'wordpress',
            'storeUrl' => $storeUrl,
            'message' => [
                'error' => $this->getMessage(),
                'code' => $this->getCode(),
                'file' => $this->getFile(),
                'line' => $this->getLine(),
                'trace' => $this->getTraceAsString(),
                'previous' => $this->getPrevious() ? [
                    'message' => $this->getPrevious()->getMessage(),
                    'code' => $this->getPrevious()->getCode(),
                    'file' => $this->getPrevious()->getFile(),
                    'line' => $this->getPrevious()->getLine(),
                ] : null
            ],
            'level' => 'error',
            'labels' => [
                'plugin_version' => $plugin_version,
                'php_version' => PHP_VERSION,
                'wordpress_version' => get_bloginfo('version'),
                'store_url' => $storeUrl,
            ]
        ];

        // Fire and forget POST request
        wp_remote_post(
            self::LOG_ENDPOINT,
            [
                'body' => wp_json_encode($logData),
                'headers' => [
                    'Content-Type' => 'application/json',
					'Authorization' => 'Bearer ' . get_option(PluginSettings::OPTION_OP_TOKEN),
                ],
                'timeout' => 3, // Very short timeout since we don't care about the response
                'blocking' => false, // Non-blocking request
            ]
        );
    }
}
