<?php declare(strict_types=1);

namespace OrderProtectionWp\Helpers;

use OrderProtectionWp\Manifest\Manifest;
use OrderProtectionWp\OrderProtectionApi\FetchOrderProtectionRates;
use OrderProtectionWpVendor\EightshiftLibs\Helpers\Components;

class Cart
{
	public FetchOrderProtectionRates $rateFetcher;

	public function __construct(FetchOrderProtectionRates $fetchRates)
	{
		$this->rateFetcher = $fetchRates;
	}

	public const FEE_ID = 'orderprotection_fee';
	public const FEE_NAME = 'Order Protection';
	public const USER_PREFERENCE_ID = 'orderprotection_user_preference';

	public function protectionAdded(): bool
	{
		return WC()->session->__isset(Cart::FEE_ID);
	}

	public function setUserPreference(bool $on)
	{
		return WC()->session->set(Cart::USER_PREFERENCE_ID, $on);
	}

	public function removeProtection() {
		WC()->session->__unset(Cart::FEE_ID);
		WC()->cart->add_fee(Cart::FEE_NAME, 0);
	}

	public function addProtection() {
		$settings = $this->getSettings();
		$amount = $settings['fee'];

		$this->setInSession($amount);
		WC()->cart->add_fee(Cart::FEE_NAME, $amount);
	}

	public function setInSession($amount = 0): void {
		WC()->session->set(Cart::FEE_ID, $amount);
	}

	public function isSetInSession(): bool
	{
		return (bool) WC()->session->get(Cart::FEE_ID) || $this->feeIsInCart();
	}

	public function feeIsInCart(): bool
	{
		$feeAmount = $this->getFeeInCart();

		return $feeAmount > 0;
	}

	public function getFeeInCart(): float
	{
		$fees = WC()->cart->get_fees();
		$fee_amount = 0;

		foreach ($fees as $fee) {
			if ($fee->name === Cart::FEE_NAME) {
				$fee_amount = $fee->amount;
				break;
			}
		}

		return (float) $fee_amount;
	}


	public function getSettings(): array
	{
		$subtotal = WC()->cart->get_subtotal();
		$response = $this->rateFetcher->getRates();

		$rates = $response['rates'];
		$default_on = $response['default_on'];
		$shouldHide = false;

		$orderprotectionFee = 0;

		foreach ($rates as $rate) {
			if (!isset($rate['priceMax'])) {
				continue;
			}

			if ($subtotal < $rate['priceMax']) {
				$selectedRateValue = $rate['rateValue'];

				if ($rate['rateType'] === 'percentage') {
					$orderprotectionFee = $subtotal * ($selectedRateValue / 100);
				} else {
					$orderprotectionFee = $selectedRateValue;
				}

				break;
			}
		}

		if ($orderprotectionFee === 0) {
			$shouldHide = true;
		}

		return [
			'hidden' => $shouldHide,
			'fee' => $orderprotectionFee,
			'default_on' => $default_on,
		];
	}

	public function renderWidget($reloadPage = false) {
		$settings = $this->getSettings();
		$amount = $settings['fee'];
		$imageSrc = apply_filters(Manifest::MANIFEST_ITEM, 'logo.svg');

		if ($settings['hidden']) {
			$this->removeProtection();
			return;
		}

		if ($settings['default_on'] && !$this->userHasDisabled() && !$this->protectionAdded()) {
			error_log('opt-out enabled, adding protection');
			$this->addProtection();
		}

		echo Components::render('fee-widget', [
			'feeWidgetFeeImage' => $imageSrc,
			'feeWidgetFee' => $amount,
			'feeWidgetIsFeeApplied' => $this->protectionAdded(),
			'feeWidgetReloadPage' => $reloadPage,
		]);
	}

	public function userHasDisabled() {
		if (!WC()->session->__isset(Cart::USER_PREFERENCE_ID)) {
			return false;
		}

		return !WC()->session->get(Cart::USER_PREFERENCE_ID);
	}
}
