<?php

namespace OrderProtectionWp\Log;

use Exception;
use OrderProtectionWp\Settings\PluginSettings;

class CloudLogger
{
    private const LOG_ENDPOINT = 'https://api.production.orderprotection.com/v1/integrations/logs/create';

    // Log level constants
    public const LEVEL_ERROR = 'error';
    public const LEVEL_INFO = 'info';

    /**
     * Log an error message with optional exception
     */
    public function error(string $message, ?Exception $exception = null, array $context = []): void
    {
        $this->sendLog(self::LEVEL_ERROR, $message, $exception, $context);
    }

    /**
     * Log an info message
     */
    public function info(string $message, array $context = []): void
    {
        $this->sendLog(self::LEVEL_INFO, $message, null, $context);
    }

    /**
     * Legacy method for backward compatibility
     */
    public function log(string $message, int $code, ?Exception $previous = null): void
    {
        $this->error($message, $previous);
    }

    /**
     * Main logging method that handles all log levels
     */
    private function sendLog(string $level, string $message, ?Exception $exception = null, array $context = []): void
    {
        $storeUrl = get_option(PluginSettings::OPTION_STORE_URL, '');

        // Get plugin data to access version - use a safer approach
        $plugin_version = $this->getPluginVersion();

        $logData = [
            'service' => 'wordpress',
            'storeUrl' => $storeUrl,
            'message' => $this->buildMessageData($message, $exception, $context),
            'level' => $level,
            'labels' => [
                'plugin_version' => $plugin_version,
                'php_version' => PHP_VERSION,
                'wordpress_version' => get_bloginfo('version'),
                'store_url' => $storeUrl,
                'woocommerce_version' => $this->getWooCommerceVersion(),
            ]
        ];

        // Fire and forget POST request
        wp_remote_post(
            self::LOG_ENDPOINT,
            [
                'body' => wp_json_encode($logData),
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . get_option(PluginSettings::OPTION_OP_TOKEN),
                ],
                'timeout' => 0.01, // Very short timeout since we don't care about the response
                'blocking' => false,
            ]
        );
    }

    /**
     * Get plugin version safely
     */
    private function getPluginVersion(): string
    {
        // Try multiple approaches to get the plugin version
        $version = 'unknown';

        // Method 1: Try using WP_PLUGIN_DIR if defined
        if (defined('WP_PLUGIN_DIR')) {
            $plugin_file = WP_PLUGIN_DIR . '/orderprotection/orderprotection.php';
            if (file_exists($plugin_file)) {
                $plugin_data = get_plugin_data($plugin_file);
                if (isset($plugin_data['Version'])) {
                    $version = $plugin_data['Version'];
                }
            }
        }

        // Method 2: Try using plugin_dir_path if the first method failed
        if ($version === 'unknown') {
            $plugin_file = plugin_dir_path(__FILE__) . '../../orderprotection.php';
            if (file_exists($plugin_file)) {
                $plugin_data = get_plugin_data($plugin_file);
                if (isset($plugin_data['Version'])) {
                    $version = $plugin_data['Version'];
                }
            }
        }

        // Method 3: Try using ABSPATH as fallback
        if ($version === 'unknown' && defined('ABSPATH')) {
            $plugin_file = ABSPATH . 'wp-content/plugins/orderprotection/orderprotection.php';
            if (file_exists($plugin_file)) {
                $plugin_data = get_plugin_data($plugin_file);
                if (isset($plugin_data['Version'])) {
                    $version = $plugin_data['Version'];
                }
            }
        }

        return $version;
    }

    /**
     * Build the message data structure based on whether an exception is provided
     */
    private function buildMessageData(string $message, ?Exception $exception = null, array $context = []): array
    {
        if ($exception !== null) {
            return [
                'error' => $exception->getMessage(),
                'code' => $exception->getCode(),
                'file' => $exception->getFile(),
                'line' => $exception->getLine(),
                'trace' => $exception->getTraceAsString(),
                'previous' => $exception->getPrevious() ? [
                    'message' => $exception->getPrevious()->getMessage(),
                    'code' => $exception->getPrevious()->getCode(),
                    'file' => $exception->getPrevious()->getFile(),
                    'line' => $exception->getPrevious()->getLine(),
                ] : null,
                'context' => $context
            ];
        }

        return [
            'message' => $message,
            'context' => $context
        ];
    }

    /**
     * Get WooCommerce version safely
     */
    private function getWooCommerceVersion(): string
    {
        // Try multiple approaches to get the WooCommerce version
        $version = 'unknown';

        // Method 1: Try using WC_VERSION if defined
        if (defined('WC_VERSION')) {
            $version = WC_VERSION;
        }

        // Method 2: Try using get_option if the first method failed
        if ($version === 'unknown') {
            $option_version = get_option('woocommerce_version');
            if ($option_version) {
                $version = $option_version;
            }
        }

        // Method 3: Try using get_site_option if the first two methods failed
        if ($version === 'unknown') {
            $site_option_version = get_site_option('woocommerce_version');
            if ($site_option_version) {
                $version = $site_option_version;
            }
        }

        return $version;
    }
}
