<?php

declare(strict_types=1);

namespace OrderProtectionWp\OrderProtectionApi;

use Exception;
use OrderProtectionWp\Settings\PluginSettings;
use OrderProtectionWp\Log\CloudLogger;

class Client
{
	private CloudLogger $logger;

    public const API_URI = 'https://api.production.orderprotection.com';

	public function __construct() {
		$this->logger = new CloudLogger();
	}

    private function createRequest(string $method, string $path, array $data = null)
    {
        $initialized = $this->verifyInitialized();

        if (!$initialized) {
            return [];
        }

        $token = get_option(PluginSettings::OPTION_OP_TOKEN);

        $response = wp_remote_request($this->createEndpoint($path), [
            'method' => $method,
            'body' => is_null($data) ? null : wp_json_encode($data),
            'headers' => [
                'Content-Type' => 'application/json',
                'Authorization' => "Bearer $token",
            ]
        ]);

        if (is_wp_error($response)) {
			throw $response;
        }

        $status_code = wp_remote_retrieve_response_code($response);

        if ($status_code > 300) {
            $response_body = wp_remote_retrieve_body($response);

            if (empty($response_body)) {
				throw new Exception('Failed to get response body');
            }

            throw new Exception("request failed with status code: $status_code, response body: $response_body");
        }

        $body = wp_remote_retrieve_body($response);
        $decoded = json_decode($body, true);

        return $decoded['data'];
    }

    private function createEndpoint($path)
    {
        return self::API_URI . $path;
    }

    private function verifyInitialized()
    {
        $live = get_option(PluginSettings::OPTION_ENABLE_OP_PLUGIN);
        $token = get_option(PluginSettings::OPTION_OP_TOKEN);

        if (empty($token)) {
            return false;
        }

        if (!$live) {
            return false;
        }

        return true;
    }

    /* This marks an order as cancelled in OrderProtection and will ensure that you are not billed for any protection costs */
    public function cancelOrder(string $orderId)
    {
        try {
            $this->createRequest('DELETE', '/v1/orders/' . $orderId);
		} catch (Exception $e) {
			$this->logger->error("unable to cancel order: $orderId", $e);

			return;
		}
    }

    public function getProduct($product_id)
    {
        try {
            return $this->createRequest('GET', '/v2/products/' . $product_id);
        } catch (Exception $e) {
			$this->logger->error("unable to get product", $e);
            return null;
        }
    }

    public function addProductToBundle($product_bundle_id, $child_id)
    {
        $data = [
            'productIds' => [strval($child_id)],
        ];

        try {
            return $this->createRequest('POST', '/v2/products/' . $product_bundle_id . '/bundle', $data);
        } catch (Exception $e) {
            error_log('This record already exists, ignoring');
			$this->logger->error("product already exists ignoring", $e);
			return;
        }
    }

    public function createProduct(array $data, $parent_id = '')
    {
        $path = !empty($parent_id) ? '/v2/products/' . $parent_id . '/children' : '/v2/products';

        try {
            return $this->createRequest('POST', $path, $data);
        } catch (Exception $e) {
			$this->logger->error("Unable to create product", $e);
			return;
        }
    }

    public function deleteProduct($product_id)
    {
        try {
            return $this->createRequest('DELETE', '/v2/products/' . $product_id);
        } catch (Exception $e) {
			$this->logger->error("Unable to delete product", $e);
			return;
        }
    }

    public function updateProduct($product_id, array $data)
    {
        try {
            return $this->createRequest('PUT', '/v2/products/' . $product_id, $data);
        } catch (Exception $e) {
			$this->logger->error("Unable to update product", $e);
			return;
        }
    }

    public function deleteProductFromBundle($product_bundle_id, $product_id)
    {
        $data = [
            'productIds' => [$product_id],
        ];

        try {
            return $this->createRequest('POST', '/v2/products/' . $product_bundle_id . '/bundle/remove', $data);
        } catch (Exception $e) {
			$this->logger->error("unable to delete product from bundle", $e);
			return;
        }
    }
}
