<?php

/**
 * The file that holds FetchOrderProtectionRates class
 *
 * @package OrderProtectionWp\OrderProtectionApi
 */

declare(strict_types=1);

namespace OrderProtectionWp\OrderProtectionApi;

use OrderProtectionWp\Settings\PluginSettings;

/**
 * Class FetchOrderProtectionRates
 *
 * This class is responsible for fetching the order protection rates from an API and returning them in a formatted array.
 *
 * @since 1.0.0
 */
class FetchOrderProtectionRates implements FetchOrderProtectionRatesInterface
{

	/**
	 * Fetch rates from the API and return an array of formatted rate data.
	 *
	 * @return array<string, mixed> The array of formatted rate data containing priceMax, rateType, and rateValue.
	 */
	public function fetchRates(): array
	{
		$ratesUrl = get_option(PluginSettings::OPTION_STORE_URL);
		$response = wp_remote_get("https://api.production.orderprotection.com/v1/quote/insurance?store_url=$ratesUrl");

		if (is_wp_error($response)) {
			return [];
		}

		$responseBody = wp_remote_retrieve_body($response);

		if (empty($responseBody)) {
			return [];
		}

		$apiRates = json_decode($responseBody, true);
		$priceRangeList = $apiRates['rules'] ?? [];

		return [
			'rates' => array_map(function ($priceRange) {
				if (!isset($priceRange['terms']['customer'])) {
					error_log('no customer amount found skipping');
					return null;
				}

				return [
					'priceMin' => $priceRange['min'],
					'priceMax' => $priceRange['max'],
					'rateType' => $priceRange['terms']['customer']['type'],
					'rateValue' => $priceRange['terms']['customer']['amount'],
				];
			}, $priceRangeList),
			'default_on' => $apiRates['automaticallyAddToCart'],
		];
	}

	/**
	 * Get the rates.
	 *
	 * @return array<string, mixed> The array of formatted rate data.
	 */
	public function getRates(): array
	{
		return $this->fetchRates();
	}

	/**
	 * Get OP fee string
	 *
	 * @return string OrderProtection fee.
	 */
	public function getOrderProtectionFee(): array
	{
		// Get the rates from the API.
		$response = $this->getRates();

		$cart = WC()->cart;

		if (is_null($cart)) {
			apply_filters('orderprotection_fee_widget_show', false);
			return apply_filters('orderprotection_fee_string', '0');
		}

		// Get the cart total.
		$cartTotal = $cart->get_cart_contents_total();
		$shouldShow = true;

		$orderprotectionFee = 0;

		foreach ($response['rates'] as $rate) {
			if (!isset($rate)) {
				$shouldShow = false;
				continue;
			}

			if ($cartTotal < $rate['priceMax']) {
				$selectedRateValue = $rate['rateValue'];

				if ($rate['rateType'] === 'percentage') {
					$orderprotectionFee = $cartTotal * ($selectedRateValue / 100);
				} else {
					$orderprotectionFee = $selectedRateValue;
				}

				break;
			}
		}

		return [
			'should_show' => (bool) $shouldShow,
			'default_on' => $response['default_on'],
			'fee_string' => (string) $orderprotectionFee,
		];
	}
}
