<?php

declare(strict_types=1);

namespace OrderProtectionWp\Resolutions;

use OrderProtectionWp\Log\CloudLogger;
use OrderProtectionWpVendor\EightshiftLibs\Services\ServiceInterface;
use OrderProtectionWp\WooCommerce\SendOrderDetailsToApi;
use OrderProtectionWp\Resolutions\Types\Resolutions;

class ClaimResolutionProcessor implements ServiceInterface
{
    private SendOrderDetailsToApi $sendOrderDetailsToApi;
    private CloudLogger $logger;

    public function __construct(SendOrderDetailsToApi $sendOrderDetailsToApi)
    {
        $this->sendOrderDetailsToApi = $sendOrderDetailsToApi;
        $this->logger = new CloudLogger();
    }

    public function register(): void
    {
        add_action('wp_ajax_nopriv_orderprotection_process_claim_approved', [$this, 'processClaimApproved'], 10, 2);
        add_action('wp_ajax_nopriv_orderprotection_process_remove_protection', [$this, 'processRemoveProtection'], 10, 2);
    }

    private function getParameters()
    {
        error_log("ClaimResolutionProcessor getParameters called");
        error_log("POST data: " . print_r($_POST, true));

        $payload = wp_unslash($_POST['payload'] ?? '');
        error_log("Raw payload: " . $payload);

        $body = json_decode($payload, true);
        error_log("JSON decode result: " . print_r($body, true));
        error_log("JSON decode error: " . json_last_error_msg());

        $orderId = wp_unslash($_POST['order_id'] ?? '');
        error_log("Order ID: " . $orderId);

        return [
            'body' => $body,
            'order_id' => $orderId,
        ];
    }

    public function processClaimApproved(): void
    {
        $params = $this->getParameters();
        $order_id = $params['order_id'];
        $body = $params['body'];

        error_log('Processing claim approval');
        $order = wc_get_order($order_id);

        if (!$order) {
            $this->logger->info("no order found for claim approval: $order_id");
            error_log("Order not found for processing claim approved: $order_id");

            return;
        }

        $resolutionManager = new ResolutionManager();
        $resolutionManager->registerStrategy(Resolutions::Refund, new RefundResolution());
        $resolutionManager->registerStrategy(Resolutions::Reship, new ReshipResolution($this->sendOrderDetailsToApi));

        error_log("Body: " . print_r($body, true));
        error_log("Resolutions: " . print_r($body['resolutions'], true));

        foreach ($body['resolutions'] as $resolution) {
            $resolution_type = $resolution['type'];
            error_log("processing resolution: $resolution_type");
            error_log("Resolution body: " . print_r($resolution, true));

            $resolutionManager->handleResolution($order, $resolution, $body['claim_id']);
        }

        $order->save();
    }

    public function processRemoveProtection(): void
    {
        $params = $this->getParameters();
        $order_id = $params['order_id'];
        $body = $params['body'];

        error_log('Processing claim approval');
        $order = wc_get_order($order_id);

        if (!$order) {
            error_log("Order not found for processing remove protection: $order_id");
            return;
        }

        $resolutionManager = new ResolutionManager();
        $resolutionManager->registerStrategy(Resolutions::RemoveProtection, new RemoveProtectionResolution());
        $resolutionManager->handleResolution($order, $body, 'N/A');

        $order->save();
    }
}
