<?php

declare(strict_types=1);

namespace OrderProtectionWp\Resolutions;

use WC_Order;
use OrderProtectionWp\WooCommerce\SendOrderDetailsToApi;

class ReshipResolution implements ResolutionStrategyInterface
{
    private SendOrderDetailsToApi $sendOrderDetailsToApi;

    public function __construct(SendOrderDetailsToApi $sendOrderDetailsToApi)
    {
        $this->sendOrderDetailsToApi = $sendOrderDetailsToApi;
    }

    public function handle(WC_Order $order, array $resolution): void
    {
        error_log("reship resolution");
        $details = $resolution['details'];
        $itemDetails = $details['items'];
        $newShippingAddress = $details['shipping_address'];

        $billingAddress = [
            'first_name' => $order->get_billing_first_name(),
            'last_name' => $order->get_billing_last_name(),
            'company' => $order->get_billing_company(),
            'address_1' => $order->get_billing_address_1(),
            'address_2' => $order->get_billing_address_2(),
            'city' => $order->get_billing_city(),
            'state' => $order->get_billing_state(),
            'postcode' => $order->get_billing_postcode(),
            'country' => $order->get_billing_country(),
            'phone' => $order->get_billing_phone()
        ];

        $shippingAddress = [
            'first_name' => $order->get_shipping_first_name(),
            'last_name' => $order->get_shipping_last_name(),
            'company' => $order->get_shipping_company(),
            'address_1' => $order->get_shipping_address_1(),
            'address_2' => $order->get_shipping_address_2(),
            'city' => $order->get_shipping_city(),
            'state' => $order->get_shipping_state(),
            'postcode' => $order->get_shipping_postcode(),
            'country' => $order->get_shipping_country(),
            'phone' => $order->get_shipping_phone()
        ];

        $reshippedOrder = wc_create_order([
            'customer_id' => $order->get_customer_id(),
        ]);

        $reshippedOrder->set_billing_address($billingAddress);
        $reshippedOrder->set_shipping_address($shippingAddress);

        if (isset($newShippingAddress)) {
            $reshippedOrder->set_shipping_address([
                'address_1' => $newShippingAddress['address1'],
                'address_2' => $newShippingAddress['address2'],
                'city' => $newShippingAddress['city'],
                'country' => $newShippingAddress['country'],
                'postcode' => $newShippingAddress['zip'],
                'state' => $newShippingAddress['state'],
                'first_name' => $order->get_shipping_first_name(),
                'last_name' => $order->get_shipping_last_name(),
            ]);
        }

        foreach ($itemDetails as $detail) {
            error_log("adding sku: " . $detail['sku']);
            $product_sku = $detail['sku'];
            $product_id = $product_sku ? wc_get_product_id_by_sku($product_sku) : ($detail['product_id'] ?? 0);
            $product = wc_get_product($product_id);

            if (!$product) {
                error_log("SKU NOT FOUND: $product_sku");
                $reshippedOrder->add_order_note("There was an issue adding $product_sku to the order\n\nPlease ensure that the item is added.");
                continue;
            }

            $order_item_id = $reshippedOrder->add_product($product, $detail['quantity']);
            $order_item = $reshippedOrder->get_item($order_item_id);

            if ($order_item) {
                $order_item->set_total(0);
                $order_item->set_subtotal(0);
                $order_item->save();
            }
        }

        $order_id = $order->get_id();

        $reshippedOrder->set_billing_email($order->get_billing_email());
        $reshippedOrder->calculate_totals();
        $reshippedOrder->set_total('0.00');
        $reshippedOrder->add_order_note("OrderProtection Replacement for: $order_id");
        $reshippedOrder->set_customer_note("OP Replacement for: $order_id");
        $reshippedOrder->set_status('processing');
        $reshippedOrder->set_created_via('OrderProtection');
        $reshippedOrder->update_meta_data('_created_via', 'OrderProtection');

        // This is to avoid this running through the changed handler.
        $reshippedOrder->update_meta_data(SendOrderDetailsToApi::OP_METADATA_TAG, true);
        $reshippedOrder->save();

        error_log("reshipped order id: " . $reshippedOrder->get_id());

        // here we will pass the original source order id on.
        // This will mark it as a reshipment in OrderProtection

        $this->sendOrderDetailsToApi->prepareOrderProtectionOrder($reshippedOrder->get_id(), $order_id);
    }
}
