<?php

/**
 * The class register route for $className endpoint
 *
 * @package OrderProtectionWp\Rest\Routes
 */

declare(strict_types=1);

namespace OrderProtectionWp\Rest\Routes;

use OrderProtectionWp\Config\Config;
use OrderProtectionWp\Settings\PluginSettings;
use OrderProtectionWpVendor\EightshiftLibs\Rest\Routes\AbstractRoute;
use OrderProtectionWpVendor\EightshiftLibs\Rest\CallableRouteInterface;
use WP_REST_Request;
use WP_REST_Response;
use WP_Error;
use OrderProtectionWp\WooCommerce\SendOrderDetailsToApi;
use OrderProtectionWp\Resolutions\Types\Resolutions;

/**
 * Class ClaimResolutionRoute
 */
class ClaimResolutionRoute extends AbstractRoute implements CallableRouteInterface
{

    public const Refund = 'refund';
    public const Reship = 'reship';
    public const Cancel = 'cancellation';

    /**
     * SendOrderDetailsToApi member variable
     *
     * @var SendOrderDetailsToApi
     */
    private SendOrderDetailsToApi $sendOrderDetailsToApi;

    public function __construct(SendOrderDetailsToApi $sendOrderDetailsToApi)
    {
        $this->sendOrderDetailsToApi = $sendOrderDetailsToApi;
    }
    /**
     * Method that returns project Route namespace.
     *
     * @return string Project namespace OrderProtectionWpVendor\for REST route.
     */
    protected function getNamespace(): string
    {
        return 'order-protection';
    }

    /**
     * Method that returns project route version.
     *
     * @return string Route version as a string.
     */
    protected function getVersion(): string
    {
        return Config::getProjectRoutesVersion();
    }

    /**
     * Get the base url of the route
     *
     * @return string The base URL for route you are adding.
     */
    protected function getRouteName(): string
    {
        return '/claim-resolution';
    }

    /**
     * Get callback arguments array
     *
     * @return array<string, mixed> Either an array of options for the endpoint, or an array of arrays for multiple methods.
     */
    protected function getCallbackArguments(): array
    {
        return [
            'methods' => static::CREATABLE,
            'callback' => [$this, 'routeCallback'],
            'permission_callback' => [$this, 'isAuthorizedFromOrderprotection']
        ];
    }

    public function isAuthorizedFromOrderprotection(WP_REST_Request $request): bool
    {
        $headers = $request->get_headers();
        $webhookSecret = $headers['x_webhook_secret'] ? ($headers['x_webhook_secret'][0] ?? null) : null;

        if (!isset($webhookSecret)) {
            error_log("secret is not set from request. Ensure your webhook has a secret set in OrderProtection dashboard");
            return false;
        }

        // TODO: perform HMAC validation

        $secret = get_option(PluginSettings::OPTION_WEBHOOK_SECRET);

        if ($secret !== $webhookSecret) {
            error_log('secrets do not match, this is not an authorized request');
            return false;
        }

        return true;
    }

    /**
     * Method that returns rest response
     *
     * @param WP_REST_Request $request Data got from endpoint url.
     *
     * @return WP_REST_Response|mixed If response generated an error, WP_Error, if response
     *                                is already an instance, WP_HTTP_Response, otherwise
     *                                returns a new WP_REST_Response instance.
     * @throws \WC_Data_Exception
     */
    public function routeCallback(WP_REST_Request $request)
    {
        $data = $request->get_body();
        $data = json_decode($data, true);
        $body = $data['body'];
        $topic = $data['topic'];

		$topics = [
			'claim.approved' => 'orderprotection_process_claim_approved',
			'protection.removed' => 'orderprotection_process_remove_protection',
		];

		error_log("Received request for topic $topic");

        if (session_status() === PHP_SESSION_ACTIVE) {
            session_write_close();
        }

		wp_remote_post(admin_url('admin-ajax.php'), [
			'body' => [
				'action' => $topics[$topic],
				'payload' => json_encode($body),
                'order_id' => intval($body['source_order_id']),
			],
            'blocking' => false,
			'timeout' => 0.01,
		]);

        wp_send_json_success([
            'success' => true,
        ], 201);
    }
}
