<?php

/**
 * The file that holds a PluginSettings class.
 *
 * @package OrderProtectionWp\Settings
 */

declare(strict_types=1);

namespace OrderProtectionWp\Settings;

use OrderProtectionWpVendor\EightshiftLibs\Services\ServiceInterface;

/**
 * PluginSettings class.
 *
 * @since 1.0.0
 */
class PluginSettings implements ServiceInterface
{
	public const OPTION_ENABLE_OP_PLUGIN = 'wc_orderprotection_enable';
	public const OPTION_OP_TOKEN = 'wc_orderprotection_api_token';
	public const OPTION_STORE_URL = 'wc_orderprotection_store_url';
	public const OPTION_WEBHOOK_SECRET = 'wc_orderprotection_webhook_secret';

	/**
	 * Register all the hooks
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function register(): void
	{
		add_filter('woocommerce_settings_tabs_array', [$this, 'addWcSettingsTab'], 50);
		add_action('woocommerce_settings_tabs_orderprotection', [$this, 'addWcTabContent']);
		add_action('woocommerce_update_options_orderprotection', [$this, 'updateWcSettings']);
	}

	/**
	 * Add OrderProtection tab in WooCommerce settings.
	 *
	 * @since 1.0.0
	 *
	 * @param array<string, string> $settingsTabs Array of setting tabs.
	 *
	 * @return array<string, string> Modified setting tabs.
	 */
	public function addWcSettingsTab(array $settingsTabs): array
	{
		$settingsTabs['orderprotection'] = esc_html__('Order Protection', 'orderprotection-wp');

		return $settingsTabs;
	}

	/**
	 * Function to add the settings to the OrderProtection Tab.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function addWcTabContent(): void
	{
		\woocommerce_admin_fields($this->getSettingConfig());
	}

	/**
	 * Function to save the settings.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function updateWcSettings(): void
	{
		\woocommerce_update_options($this->getSettingConfig());
		do_action('sync_orderprotection_products');
	}

	/**
	 * Function to determine if OrderProtection WooCommerce option is enabled.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True or false based on the saved option.
	 */
	public static function isOrderprotectionActive(): bool
	{
		return get_option(self::OPTION_ENABLE_OP_PLUGIN) === 'yes';
	}


	/**
	 * Array of WooCommerce Settings for OrderProtection.
	 *
	 * @since 1.0.0
	 *
	 * @return array<array<string, string>> Returns the settings as an array.
	 */
	private function getSettingConfig(): array
	{
		return [
			[
				'name' => esc_html__('Order Protection', 'orderprotection-wp'),
				'type' => 'title',
				'desc' => __(
					"Turn shipping problems into a new revenue stream. <br> With Order Protection, take your shipping challenges from problem to profits. <br>We'll handle shipping issues 24/7 for your customers while you enjoy more revenue and better customer loyalty.", // phpcs:ignore Generic.Files.LineLength.TooLong
					'orderprotection-wp'
				),
				'id' => 'wc_settings_tab_demo_section_title',
			],
			[
				'name' => esc_html__('Enable Order Protection', 'orderprotection-wp'),
				'type' => 'checkbox',
				'default' => 'yes',
				'desc' => esc_html__('Enable Order Protection', 'orderprotection-wp'),
				'id' => self::OPTION_ENABLE_OP_PLUGIN,
				'desc_tip' => esc_html__(
					'Enable/ Disable Order Protection without needing to deactivate the plugin.',
					'orderprotection-wp'
				),
			],
			[
				'name' => esc_html__('Store URL', 'orderprotection-wp'),
				'type' => 'text',
				'desc' => esc_html__('Add the specific store URL', 'orderprotection-wp'),
				'id' => self::OPTION_STORE_URL,
			],
			[
				'name' => esc_html__('API access token', 'orderprotection-wp'),
				'type' => 'password',
				'desc' => esc_html__('Add the Order Protection API access token', 'orderprotection-wp'),
				'id' => self::OPTION_OP_TOKEN,
				'desc_tip' => esc_html__(
					'Add the API token generated on the Order Protection backend, used for the API authentication.',
					'orderprotection-wp'
				),
			],
			[
				'name' => esc_html__('Webhook Secret', 'orderprotection-wp'),
				'type' => 'password',
				'desc' => esc_html__('Add your secret that was created with your webhook.', 'orderprotection-wp'),
				'id' => self::OPTION_WEBHOOK_SECRET,
				'desc_tip' => esc_html__(
					'Use this to protect your webhook route from unauthorized access',
					'orderprotection-wp'
				),
			],
			[
				'type' => 'sectionend',
				'id' => 'wc_settings_tab_demo_section_end',
			],
		];
	}
}
