<?php

namespace OrderProtectionWp\WooCommerce;

use WC_Order;
use Exception;
use OrderProtectionWp\Log\CloudLogger;

class RefundService
{
    public function __construct()
    {
        $this->logger = new CloudLogger();
    }

    private CloudLogger $logger;
    /**
     * Process a refund for a WooCommerce order.
     *
     * @param WC_Order $order The WooCommerce order to refund.
     * @param float $requestedAmount The amount to refund.
     * @param string $reason The reason for the refund.
     * @param array $lineItems Optional line items to refund (for partial refunds).
     * @param bool $restockItems Whether to restock refunded items.
     * @throws Exception If refund fails or inputs are invalid.
     * @return bool True if refund is successful.
     */
    public function processRefund(
        WC_Order $order,
        float $requestedAmount,
        string $reason = '',
        array $lineItems = [],
        bool $restockItems = false
    ): bool {
        if (!$order instanceof WC_Order) {
            $this->logger->error("invalid instance of order provided");
            return false;
        }

        if ($requestedAmount <= 0) {
            $this->logger->error("invalid refund amount", null, [
                'order_id' => $order->get_id(),
                'refund_amount' => $requestedAmount,
            ]);

            return false;
        }

        $orderTotal = (float) $order->get_total();
        $refundAmount = min($requestedAmount, $orderTotal);
        $refundAmount = round($refundAmount, wc_get_price_decimals());

        error_log("Processing refund for order ID {$order->get_id()} for amount: {$refundAmount}");

        $paymentGateway = wc_get_payment_gateway_by_order($order);

        if ($paymentGateway && $paymentGateway->supports('refunds')) {
            $result = $paymentGateway->process_refund(
                $order->get_id(),
                $refundAmount,
                $reason
            );

            if (is_wp_error($result)) {
                error_log("Gateway refund failed: " . $result->get_error_message());

                $this->logger->error("Gateway refund failed: " . $result->get_error_message(), null, [
                    'order_id' => $order->get_id(),
                    'refund_amount' => $refundAmount,
                ]);

                return false;
            }

            error_log("Gateway refund processed successfully.");
        } else {
            error_log("Gateway does not support refunds, creating WooCommerce refund record.");

            $this->logger->error("Gateway does not support refunds", null, [
                'order_id' => $order->get_id(),
                'refund_amount' => $refundAmount,
            ]);

            $refundArgs = [
                'amount' => $refundAmount,
                'reason' => $reason,
                'order_id' => $order->get_id(),
                'line_items' => $lineItems,
                'restock_items' => $restockItems,
            ];

            $refund = wc_create_refund($refundArgs);

            if (is_wp_error($refund)) {
                error_log("WooCommerce refund creation failed: " . $refund->get_error_message());

                $this->logger->error("WooCommerce refund creation failed " . $refund->get_error_message(), null, [
                    'order_id' => $order->get_id(),
                    'refund_amount' => $refundAmount,
                ]);

                return false;
            }
        }

        $this->updateOrderStatus($order, $refundAmount, $reason);
        return true;
    }

    /**
     * Update the order status and add a refund note.
     *
     * @param WC_Order $order The WooCommerce order.
     * @param float $refundAmount The refund amount.
     * @param string $reason The refund reason.
     */
    private function updateOrderStatus(WC_Order $order, float $refundAmount, string $reason): void
    {
        $orderTotal = (float) $order->get_total();
        $currencySymbol = get_woocommerce_currency_symbol($order->get_currency());

        // Check if the status 'partially-refunded' exists, fall back to 'processing' if not
        $newStatus = ($refundAmount >= $orderTotal) ? 'refunded' : 'processing';
        $availableStatuses = wc_get_order_statuses();

        if ($refundAmount < $orderTotal && isset($availableStatuses['wc-partially-refunded'])) {
            $newStatus = 'partially-refunded';
        }

        $note = ($refundAmount >= $orderTotal)
            ? sprintf("Full refund of %s%s issued. %s", $currencySymbol, $refundAmount, $reason)
            : sprintf("Partial refund of %s%s issued. %s", $currencySymbol, $refundAmount, $reason);

        $order->update_status($newStatus, $note);
        $order->save();
    }
}
