<?php

/**
 * The file that holds a OrderCompleteAction class.
 *
 * @package OrderProtectionWp\WooCommerce
 */

declare(strict_types=1);

namespace OrderProtectionWp\WooCommerce;

use Exception;
use OrderProtectionWp\Settings\PluginSettings;
use OrderProtectionWpVendor\EightshiftLibs\Services\ServiceInterface;
use OrderProtectionWp\Api\Dto\OrderCreateDto;
use OrderProtectionWp\Api\Dto\AddressDto;
use OrderProtectionWp\Api\Dto\OrderItemDto;
use OrderProtectionWp\Helpers\Cart;
use OrderProtectionWp\OrderProtectionApi\CreateOrder;
use DateTimeZone;
use OrderProtectionWp\Log\CloudLogger;

/**
 * OrderCompleteAction class.
 */
class SendOrderDetailsToApi implements ServiceInterface
{
	private CloudLogger $logger;

    const OP_METADATA_TAG = '_op_sent_to_api';

    /**
     * Create order member variable
     *
     * @var Creatable
     */
    private CreateOrder $createOrder;

    /**
     * Class dependencies
     *
     * @param CreateOrder $createOrder Create order dependency.
     */
    public function __construct(CreateOrder $createOrder)
    {
		$this->logger = new CloudLogger();
        $this->createOrder = $createOrder;
    }

    /**
     * Register all the hooks
     *
     * @return void
     */
    public function register(): void
    {
        if (!PluginSettings::isOrderprotectionActive()) {
            return;
        }

        add_action('woocommerce_order_status_changed', [$this, 'prepareOrder'], 50, 1);
        add_action('wp_ajax_nopriv_orderprotection_send_order_details', [$this, 'handleWpAjaxApiRequest'], 50, 1);
    }

	public function prepareOrder($orderId)
    {
        $order = wc_get_order($orderId);
        $to = $order->get_status();

        if ($order->get_meta(self::OP_METADATA_TAG)) {
            error_log("order already sent to api");
            return;
        }

        if (in_array($to, ['processing', 'completed'], true)) {
            error_log("sending to api");

			wp_remote_post(admin_url('admin-ajax.php'), [
				'body' => [
					'action' => 'orderprotection_send_order_details',
					'order_id' => $orderId,
				],
				'blocking' => false,
				'timeout' => 0.01,
			]);
        }
    }

	public function handleWpAjaxApiRequest()
	{
		$this->sendToOrderProtectionApi($_POST['order_id']);
	}

    public function sendToOrderProtectionApi($orderId)
    {
        try {
            // this will avoid sending twice.
            $order = wc_get_order($orderId);

            if (!$order) {
				error_log("order not found with $orderId");
				$this->logger->error("unable to find order with id: $orderId");

				return;
            }

            $to = $order->get_status();

            error_log("Order is in [$to] state");

            if ($order->get_meta(self::OP_METADATA_TAG)) {
				$this->logger->info("Order already has been sent to OP: $orderId");

                error_log("FOUND");
                error_log("metadata found, skipping sending this to API");
                return;
            }

            // processing means that the order has been paid for
            if (in_array($to, ['processing', 'completed'], true)) {
                error_log("sending to api");
                $this->prepareOrderProtectionOrder($orderId);
            }

			return;
		} catch (Exception $e) {
			error_log("[OrderProtection] unable to send to op platform: " . $e->getMessage());
			$this->logger->error($e->getMessage(), $e);

			return;
		}
    }

    public function log(string $message): void
    {
        error_log("[OrderProtection]: $message");
    }

    /**
     * Call OrderProtection API before a new order is made.
     *
     * We'll probably have to use background jobs for processing the calls,
     * so as not to throttle the API.
     *
     * @param int $orderId Order ID.
     *
     * @throws \Exception Exception in the case of an error.
     *
     * @return void
     */
    public function prepareOrderProtectionOrder($orderId, $originalSourceOrderId = null): void
    {
        $this->log('prepareOrderProtectionOrder');

        $order = wc_get_order($orderId);

        if (!$order) {
            $this->log('no order found');
			$this->logger->info("no order found with id: $orderId");

            return;
        }

        $feeAmount = 0;

        foreach ($order->get_items('fee') as $feeItem) {
            if ($feeItem->get_name() === Cart::FEE_NAME) {
                $this->log('customer paid for order protection adding to order details');
                $feeAmount = $feeItem->get_total() ?? 0;
                break;
            }
        }

        $lineItems = [];
        foreach ($order->get_items() as $item) {
            $product = $item->get_product();
            if (!$product) {
                continue;
            }

            $productId = $item->get_product_id();
            $imageUrl = wp_get_attachment_image_src(get_post_thumbnail_id($productId), 'single-post-thumbnail')[0] ?? '';
            $discount = $item->get_subtotal() - $item->get_total();

            $lineItems[] = new OrderItemDto(
                $item->get_name(),
                get_permalink($productId),
                $imageUrl,
                $product->get_sku(),
                (int) $item->get_quantity(),
                (float) $product->get_price(),
                (string) $item->get_id(),
                (string) $productId,
                (string) ($item->get_variation_id() ?: $productId),
                (float) $item->get_subtotal_tax(),
                (float) $discount,
            );
        }

        $createdDate = $order->get_date_created();
        $modifiedDate = $order->get_date_modified();

        $billingAddress = new AddressDto(
            $order->get_billing_first_name(),
            $order->get_billing_last_name(),
            $order->get_billing_address_1(),
            $order->get_billing_address_2(),
            $order->get_billing_city(),
            $order->get_billing_state(),
            $order->get_billing_postcode(),
            $order->get_billing_country(),
            'BILLING'
        );

        $shippingAddress = new AddressDto(
            $order->get_shipping_first_name(),
            $order->get_shipping_last_name(),
            $order->get_shipping_address_1(),
            $order->get_shipping_address_2(),
            $order->get_shipping_city(),
            $order->get_shipping_state(),
            $order->get_shipping_postcode(),
            $order->get_shipping_country(),
            'SHIPPING'
        );

        $createdAtUTC = $createdDate->setTimezone(new DateTimeZone('UTC'));
        $modifiedAtUTC = $modifiedDate->setTimezone(new DateTimeZone('UTC'));

        $orderData = new OrderCreateDto(
            (string) $order->get_id(),
            (string) $order->get_order_number(),
            $createdAtUTC->format('Y-m-d\TH:i:s.v\Z'),
            $modifiedAtUTC->format('Y-m-d\TH:i:s.v\Z'),
            'woocommerce',
            (float) $feeAmount,
            $order->get_currency(),
            (float) $order->get_total(),
            (float) $order->get_total_discount(),
            (float) $order->get_shipping_total(),
            (float) $order->get_total_tax(),
            (float) $order->get_subtotal(),
            $order->get_billing_email(),
            trim($order->get_billing_first_name() . ' ' . $order->get_billing_last_name()),
            $order->get_billing_phone() ?? '',
            $lineItems,
            [$billingAddress, $shippingAddress],
            $originalSourceOrderId,
        );


        try {
            $response = $this->createOrder->create($orderData->toArray());
            $responseBody = wp_remote_retrieve_body($response);
            $data = json_decode($responseBody, true);

            if (!empty($data['status']) && $data['status'] === 'ok') {
                $this->log('Successfully created order');

                $order->update_meta_data(self::OP_METADATA_TAG, true);
                $order->save();
            } else {
				$this->logger->error("unable to create order in op", null, [
					'order_id' => $orderId,
					'data' => $orderData->toArray(),
				]);

				return;
            }

            $message = $data['message'] ?? esc_html__('Empty response body', 'order-protection-wp');
            $status = $data['error'] ?? esc_html__('Empty response', 'order-protection-wp');
            $code = $data['statusCode'] ?? 404;

            if ($code < 200 || $code >= 300) {
                $this->log("Error $code. $status: $message");
				$this->logger->error("status code was not 200", null, [
					'order_id' => $orderId,
					'data' => $orderData->toArray(),
				]);

				return;
            }
        } catch (Exception $e) {
			$this->logger->error("Error sending order to api: $orderId", $e);
			error_log("Error sending order to api: $orderId");
			error_log($e->getMessage());

			return;
        }
    }
}
