<?php declare(strict_types=1);

/**
 * The file that holds a OrderCompleteAction class.
 *
 * @package OrderProtectionWp\WooCommerce
 */

namespace OrderProtectionWp\WooCommerce;

use OrderProtectionWpVendor\EightshiftLibs\Services\ServiceInterface;
use OrderProtectionWp\Api\Dto\ProductCreateDto;
use OrderProtectionWp\OrderProtectionApi\Client;
use Exception;
use OrderProtectionWp\Log\CloudLogger;

class SendProductDetailsToApi implements ServiceInterface
{
    private CloudLogger $logger;
    private const IMAGE_PLACEHOLDER_URL = "https://png.pngtree.com/png-vector/20210604/ourmid/pngtree-gray-network-placeholder-png-image_3416659.jpg";

    /**
     * Create order member variable
     *
     *
     */
    private Client $client;
    /**
     * Class dependencies
     *
     * @param Creatable $createOrder Create order dependency.
     */
    public function __construct(Client $client)
    {
        $this->logger = new CloudLogger();
        $this->client = $client;
    }

    /**
     * Register all the hooks
     *
     * @return void
     */
    public function register(): void
    {
        # if (!PluginSettings::isOrderprotectionActive()) {
        #   var_dump("THIS IS NOT ACTIVE");
        #   return;
        # }

        add_action('woocommerce_new_product', [$this, 'prepareOrderProtectionProduct']);
        add_action('woocommerce_update_product', [$this, 'updateOrderProtectionProduct']);
        add_action('woocommerce_delete_product', [$this, 'deleteProduct']);
        add_action('woocommerce_grouped_product_added_product', [$this, 'addGroupedProduct']);
        add_action('woocommerce_grouped_product_removed_product', [$this, 'removeGroupedProduct']);
        add_action('woocommerce_new_product_variation', [$this, 'addNewVariation']);
        add_action('woocommerce_update_product_variation', [$this, 'updateOrderProtectionProduct']);
        add_action('woocommerce_delete_product_variation', [$this, 'deleteProduct']);
    }

    public function deleteProduct($product_id)
    {
        return $this->client->deleteProduct($product_id);
    }

    public function addNewVariation(string $variation_id)
    {
        $product = wc_get_product($variation_id);
        $parent_id = $product->get_parent_id();

        return $this->createSimpleProduct($product, $parent_id);
    }

    public function updateOrderProtectionProduct($productId)
    {
        $wcProduct = wc_get_product($productId);

        if (!$wcProduct) {
            return;
        }

        $opProduct = $this->client->getProduct($productId);

        // if we have not created this product in OrderProtection we'll instead create it.
        if (!$opProduct) {
            return $this->prepareOrderProtectionProduct($productId);
        }

        $description = preg_replace('/<img[^>]*>/', '', $wcProduct->get_description());
        $sku = $wcProduct->get_sku();

        $updateBody = [
            'name' => $wcProduct->get_name(),
            'inventoryQuantity' => intval($wcProduct->get_stock_quantity()),
            'inStock' => $wcProduct->get_stock_status() === 'instock',
            'status' => $wcProduct->get_status() === 'publish' ? 'ACTIVE' : 'DRAFT',
            'description' => !empty($description) ? $description : 'No description provided',
            'price' => floatval($wcProduct->get_price()),
            'productImage' => $this->getImageSource($wcProduct),
            'sku' => !empty($sku) ? $sku : strval($wcProduct->get_id()),
        ];

        $this->client->updateProduct($productId, $updateBody);

        if (!$wcProduct->is_type('grouped')) {
            foreach ($wcProduct->get_children() as $child) {
                $this->updateOrderProtectionProduct($child);
            }
        }


        if ($wcProduct->is_type('grouped')) {
            foreach ($wcProduct->get_children() as $child) {
                $this->addGroupedProduct($productId, $child);
            }
        }
    }

    private function ensureProductExistsInOp($product_id)
    {
        $product = $this->client->getProduct($product_id);

        if (!is_null($product) && isset($product)) {
            return true;
        }

        $wcProduct = wc_get_product($product_id);
        $result = $this->prepareOrderProtectionProduct($wcProduct);

        return $result['success'];
    }

    public function addGroupedProduct($parent_id, $product_id)
    {
        $this->ensureProductExistsInOp($product_id);

        return $this->client->addProductToBundle($parent_id, $product_id);
    }

    public function removeGroupedProduct($parent_id, $product_id)
    {
        return $this->client->deleteProductFromBundle($parent_id, $product_id);
    }

    private function createVariableProduct($wcProduct)
    {
        $parentResult = $this->createSimpleProduct($wcProduct);

        if (!$parentResult['success']) {
            error_log(json_encode($parentResult['error']));
            return ['success' => true];
        }

        $children = $wcProduct->get_children();

        foreach ($children as $child_id) {
            $child_product = wc_get_product($child_id);
            $childResult = $this->createSimpleProduct($child_product);

            if (!$childResult['success']) {
                error_log($childResult['error']);
            }
        }

        return $parentResult;
    }

    private function getImageSource($wcProduct)
    {
        $image_id = $wcProduct->get_image_id();
        $image_src = wp_get_attachment_image_src($image_id, 'full');
        $image_url = $image_src ? $image_src[0] : self::IMAGE_PLACEHOLDER_URL;

        return $image_url;
    }

    private function createSimpleProduct($wcProduct, $bundleProductIds = array())
    {
        $image_url = $this->getImageSource($wcProduct);
        $parent_id = $wcProduct->get_parent_id();
        $sku = $wcProduct->get_sku();

        if (empty($sku) || is_null($sku)) {
            $sku = strval($wcProduct->get_id());
        }

        $productDetails = new ProductCreateDto(
            $wcProduct->get_id(),
            null,
            $wcProduct->get_name(),
            $sku,
            $wcProduct->get_price(),
            0,
            'SIMPLE', // we will only track instock and out of stock status. We will not track quantity.
            $wcProduct->get_stock_quantity() ?? 0,
            $wcProduct->get_stock_status() === 'instock',
            $image_url,
            $wcProduct->get_description(),
            $wcProduct->get_type(),
            $wcProduct->get_status(),
            $bundleProductIds,
        );

        return $this->createProductWrapper($productDetails->toArray(), $parent_id);
    }


    private function createProductWrapper($data, $parent_id = '')
    {
        try {
            return ['success' => true, 'result' => $this->client->createProduct($data, $parent_id)];
        } catch (Exception $e) {
            $this->logger->error("Unable to create product", $e, [
                'data' => $data,
            ]);

            return ['success' => false, 'error' => $e->getMessage()];
        }
    }

    private function createGroupedProduct($product)
    {
        $id = $product->get_id();
        $this->createSimpleProduct($product);

        foreach ($product->get_children() as $child) {
            $this->addGroupedProduct($id, $child);
        }
    }

    /**
     * Call OrderProtection API for each new product created / updated
     * this will allow us to keep inventory status in sync and allow OP to add products to reship resolutions
     * if needed.
     *
     * @throws Exception Exception in the case of an error.
     *
     * @return void
     */
    public function prepareOrderProtectionProduct($productId): void
    {
        $wcProduct = wc_get_product($productId);

        if (!$wcProduct) {
            return;
        }

        $result = null;
        $productType = $wcProduct->get_type();

        error_log($productType);

        switch ($productType) {
            // TODO: handle bundles / groups
            case 'variation':
            case 'simple':
                $result = $this->createSimpleProduct($wcProduct);
                break;
            case 'variable':
                $result = $this->createVariableProduct($wcProduct);
                break;
            case 'external':
            case 'grouped':
                $result = $this->createGroupedProduct($wcProduct);
                return;
            default:
                throw new Exception('We do not understand ' . $productType . ' as a product type.');
        }

        if (!$result['success']) {
            // show
            error_log($result['error']);
        }
    }
}
