<?php

/**
 * Class that registers WP-CLI command initial setup of theme project.
 *
 * @package EightshiftLibs\Cli
 */

declare(strict_types=1);

namespace OrderProtectionWpVendor\EightshiftLibs\Init;

use OrderProtectionWpVendor\EightshiftLibs\AdminMenus\AdminReusableBlocksMenuCli;
use OrderProtectionWpVendor\EightshiftLibs\AdminMenus\ReusableBlocksHeaderFooterCli;
use OrderProtectionWpVendor\EightshiftLibs\Cli\AbstractCli;
use OrderProtectionWpVendor\EightshiftLibs\Cli\ParentGroups\CliInit;
use OrderProtectionWpVendor\EightshiftLibs\Config\ConfigCli;
use OrderProtectionWpVendor\EightshiftLibs\Enqueue\Admin\EnqueueAdminCli;
use OrderProtectionWpVendor\EightshiftLibs\Enqueue\Blocks\EnqueueBlocksCli;
use OrderProtectionWpVendor\EightshiftLibs\Enqueue\Theme\EnqueueThemeCli;
use OrderProtectionWpVendor\EightshiftLibs\Main\MainCli;
use OrderProtectionWpVendor\EightshiftLibs\Manifest\ManifestCli;
use ReflectionClass;

/**
 * Class InitThemeCli
 */
class InitThemeCli extends AbstractCli
{
	/**
	 * All classes for initial theme setup for project.
	 *
	 * @var array<int, mixed>
	 */
	public const COMMANDS = [
		[
			'type' => 'sc',
			'label' => 'Setting service classes:',
			'items' => [
				ConfigCli::class,
				MainCli::class,
				ManifestCli::class,
				EnqueueAdminCli::class,
				EnqueueBlocksCli::class,
				EnqueueThemeCli::class,
				AdminReusableBlocksMenuCli::class,
				ReusableBlocksHeaderFooterCli::class,
			],
		],
		[
			'type' => 'blocks',
			'label' => '',
			'items' => [
				InitBlocksCli::class,
			],
		],
	];

	/**
	 * Get WP-CLI command parent name.
	 *
	 * @return string
	 */
	public function getCommandParentName(): string
	{
		return CliInit::COMMAND_NAME;
	}

	/**
	 * Get WP-CLI command name.
	 *
	 * @return string
	 */
	public function getCommandName(): string
	{
		return 'theme';
	}

	/**
	 * Define default arguments.
	 *
	 * @return array<string, int|string|boolean>
	 */
	public function getDefaultArgs(): array
	{
		return [
			AbstractCli::THEME_NAME_ARG => 'Boilerplate',
		];
	}

	/**
	 * Get WP-CLI command doc.
	 *
	 * @return array<string, array<int, array<string, bool|string>>|string>
	 */
	public function getDoc(): array
	{
		return [
			'shortdesc' => 'Kickstart your WordPress theme with this simple command.',
			'synopsis' => [
				[
					'type' => 'assoc',
					'name' => AbstractCli::THEME_NAME_ARG,
					'description' => 'Define theme name.',
					'optional' => true,
					'default' => $this->getDefaultArg(AbstractCli::THEME_NAME_ARG),
				],
			],
			'longdesc' => $this->prepareLongDesc("
				## USAGE

				Generates initial theme setup with all files to create a custom theme.

				## EXAMPLES

				# Setup theme:
				$ wp {$this->commandParentName} {$this->getCommandParentName()} {$this->getCommandName()}
			"),
		];
	}

	/* @phpstan-ignore-next-line */
	public function __invoke(array $args, array $assocArgs)
	{
		$groupOutput = $assocArgs['groupOutput'] ?? false;

		if (!$groupOutput) {
			$this->getIntroText();
		}

		$themeName = $this->getArg($assocArgs, AbstractCli::THEME_NAME_ARG);
		if ($themeName) {
			unset($assocArgs[AbstractCli::THEME_NAME_ARG]);
		}

		foreach (static::COMMANDS as $item) {
			$label = $item['label'] ?? '';
			$items = $item['items'] ?? [];
			$type = $item['type'] ?? '';

			if ($label) {
				$this->cliLog($label, 'C');
			}

			if ($items) {
				foreach ($items as $className) {
					$reflectionClass = new ReflectionClass($className);
					$class = $reflectionClass->newInstanceArgs([$this->commandParentName]);

					$class->__invoke([], \array_merge(
						$assocArgs,
						[
							'groupOutput' => $type === 'blocks',
							'introOutput' => false,
							AbstractCli::PROJECT_NAME_ARG => $themeName,
						]
					));
				}
			}

			$this->cliLog("\n");
		}

		if (!$groupOutput) {
			$this->cliLogAlert('All the files have been copied, you can start working on your awesome theme!\n\nRun `npm start` to build all the assets.', 'success', \__('Ready to go!', 'eightshift-libs'));
			$this->cliLogAlert('If you want to set up the default header and footer, run `wp boilerplate init header-footer` after building assets.', 'info', \__('Note', 'eightshift-libs'));
		}
	}
}
